/**
 * @author      MaDa-Network
 * @copyright	2015 - MaDa-Network <http://mada-network.de>
 * @license	CC BY-ND 4.0 <http://creativecommons.org/licenses/by-nd/4.0/deed.de>
 * @package	de.madanetwork.lotto
 */

WCF.Lotto = {};

WCF.Lotto.Counter = Class.extend({
    _nextDrawCount: 0,
    _executer: null,
    init: function(nextDrawCount) {
        this._nextDrawCount = nextDrawCount;

        this._counter();

        this._executer = new WCF.PeriodicalExecuter($.proxy(this._counter, this), 999);
    },
    component: function(x, v) {
        return Math.floor(x / v);
    },
    _counter: function() {
        this._nextDrawCount--;

        var $hour = this.component(this._nextDrawCount, (60 * 60)) % 24;
        var $minute = this.component(this._nextDrawCount, 60) % 60;
        var $second = this.component(this._nextDrawCount, 1) % 60;

        if ($hour < 10) {
            $hour = "0" + $hour;
        }

        if ($minute < 10) {
            $minute = "0" + $minute;
        }

        if ($second < 10) {
            $second = "0" + $second;
        }

        if (this._nextDrawCount < "1") {
            this._executer.stop();
        }

        $('.jsLottoCounter').find('div.lottoCounterHour').text($hour);
        $('.jsLottoCounter').find('div.lottoCounterMinute').text($minute);
        $('.jsLottoCounter').find('div.lottoCounterSecond').text($second);
    }
});

WCF.Lotto.Ticket = Class.extend({
    _dialog: null,
    _proxy: null,
    _jCoins: 0,
    _ticketPrice: 0,
    _priceAdditionalField: 0,
    _disabledTickets: [],
    _selectedTickets: [],
    _activTicket: 0,
    _currentField: [],
    _disabledSubmit: false,
    init: function(jCoins, ticketPrice, priceAdditionalField) {
        this._jCoins = jCoins;
        this._ticketPrice = ticketPrice;
        this._priceAdditionalField = priceAdditionalField;

        this._proxy = new WCF.Action.Proxy({
            success: $.proxy(this._success, this)
        });

        this._setDisableTickets();

        var self = this;
        $('.jsLottoField').each(function(index, field) {
            var $field = $(field);
            var $number = $field.data('number');

            if ($.inArray($number, self._disabledTickets) !== -1) {
                $field.addClass('fieldDisabled').removeAttr("title");
            } else {
                $field.click($.proxy(self._click, self));
            }
        });

        $('.jsLottoTicketFieldShuffle').each(function(index, button) {
            var $button = $(button);
            $button.click($.proxy(self._addShuffleField, self));
        });
        $('.jsLottoTicketFieldRemove').click($.proxy(this._removeAllTicketFields, this));
        $('#jsLottoSubmit').attr('disabled', 'disabled').click($.proxy(this._submitButton, this));

        this._checkedShuffleButtonField();
    },
    _removeAllTicketFields: function() {
        this._selectedTickets = [];

        for (var i = 1; i <= 12; i++) {
            $("div[data-block='" + i + "']").each(function(index, field) {
                var $field = $(field);

                if ($field.hasClass("selectedField")) {
                    $field.removeClass("selectedField");
                }
            });
        }

        $('#jsLottoSubmit').attr('disabled', 'disabled');

        this._checkedShuffleButtonField();
    },
    _setDisableTickets: function() {
        if (this._jCoins < this._ticketPrice) {
            for (var i = 1; i <= 12; i++) {
                this._disabledTickets.push(i);
            }

            var $notification = new WCF.System.Notification(WCF.Language.get('wcf.lotto.error.noCoins'), 'error');
            $notification.show(null, 10000);
        } else {
            var $jCoins = this._jCoins - this._ticketPrice;
            var count = $jCoins / this._priceAdditionalField;

            count = Math.floor(count);

            for (var i = 12; count + 1 < i; i--) {
                if (i !== 1) {
                    this._disabledTickets.push(i);
                }
            }
        }
    },
    _click: function(ticket) {
        var $ticket = $(ticket.currentTarget);
        this._dialog = $('#lottoBlockField');

        this._activTicket = $ticket.data("number");
        this._currentField = [];

        var self = this;
        $('.jsLottoBlockFields').each(function(index, field) {
            var $field = $(field);
            if ($field.hasClass("selectedField")) {
                $field.removeClass("selectedField");
            }

            $field.unbind('click').click($.proxy(self._clickField, self));
        });

        this._updateBlockInfoCount(0);

        $('.jsLottoBlockSubmitt :submit').click($.proxy(this._submitFieldBlock, this));
        $('.jsLottoBlockSubmitt :reset').click($.proxy(this._resetBlockField, this));
        $('.jsLottoBlockShuffle').click($.proxy(this._randomBlockField, this));

        this._dialog.wcfDialog({
            title: WCF.Language.get('wcf.lotto.add.block.title') + $ticket.data("number")
        });
    },
    _clickField: function(field) {
        var $field = $(field.currentTarget);
        var $number = $field.data("number");

        if ($field.hasClass("selectedField") || this._currentField.length === 6) {
            $field.removeClass("selectedField");
        } else {
            $field.addClass("selectedField");
        }

        if ($.inArray($number, this._currentField) !== -1 || this._currentField.length === 6) {
            this._currentField = $.grep(this._currentField, function(value) {
                return value !== $number;
            });
        } else {
            this._currentField.push($number);
        }

        this._updateBlockInfoCount(this._currentField.length);

        if (this._disabledSubmit) {
            if (this._currentField.length < 1 || this._currentField.length > 5) {
                $('.jsLottoBlockSubmitt :submit').removeAttr('disabled');
                this._disabledSubmit = false;
            }
        } else {
            if (this._currentField.length > 0 && this._currentField.length < 6) {
                $('.jsLottoBlockSubmitt :submit').attr('disabled', 'disabled');
                this._disabledSubmit = true;
            }
        }
    },
    _addShuffleField: function(button) {
        var $button = $(button.currentTarget);
        var $count = $button.data("id");

        var self = this;
        for (var i = 0; i < $count; i++) {
            if (this._selectedTickets.length > 0) {
                $.each(this._selectedTickets, function(index, value) {
                    if (index === 0)
                        return;

                    if (typeof self._selectedTickets[index] !== "undefined") {
                        if (self._selectedTickets[index].length === 6) {
                            self._activTicket = index + 1;
                            return;
                        } else {
                            self._activTicket = index;
                        }
                    } else {
                        self._activTicket = index;
                        return false;
                    }
                });

                if ($.inArray(self._activTicket, self._disabledTickets) !== -1 || self._activTicket > 12) {
                    return false;
                }
            } else {
                self._activTicket = 1;
            }

            self._selectedTickets[self._activTicket] = [];
            var $current = self._selectedTickets[self._activTicket];

            while ($current.length < 6) {
                var $newNumber = Math.floor((Math.random() * 49) + 1);
                if ($.inArray($newNumber, $current) === -1) {
                    $current.push($newNumber);
                }
            }

            this._updateTickets();
        }
        this._checkedShuffleButtonField();
        this._checkedSubmitButton();
    },
    _checkedShuffleButtonField: function() {
        var $countActivField = 0;

        var self = this;
        $.each(this._selectedTickets, function(index, value) {
            if (index === 0)
                return;
            if (typeof self._selectedTickets[index] === "undefined")
                return;
            if (self._selectedTickets[index].length < 6)
                return;

            $countActivField++;
        });

        var $countDisabledField = this._disabledTickets.length;
        var $freeFields = 12 - $countDisabledField - $countActivField;

        $('.jsLottoTicketFieldShuffle').each(function(index, button) {
            var $button = $(button);
            var $buttonID = $button.data("id");
            var $clickButton = $button.find("button");
            var $selectDisabled = false;

            if ($freeFields < $buttonID) {
                $selectDisabled = true;
            }

            if ($selectDisabled) {
                $clickButton.attr('disabled', 'disabled');
            } else {
                $clickButton.removeAttr('disabled');
            }
        });
    },
    _submitFieldBlock: function() {
        if (typeof this._selectedTickets[this._activTicket] === "undefined" || this._selectedTickets[this._activTicket].length > 0) {
            this._selectedTickets[this._activTicket] = [];
        }

        var $selectTickets = this._selectedTickets[this._activTicket];

        $.each(this._currentField, function(index, value) {
            $selectTickets.push(value);
        });

        this._updateTickets();
        this._checkedShuffleButtonField();
        this._checkedSubmitButton();

        this._dialog.wcfDialog('close');
    },
    _randomBlockField: function() {
        this._currentField = [];

        var self = this;
        $('.jsLottoBlockFields').each(function(index, field) {
            var $field = $(field);
            if ($field.hasClass("selectedField")) {
                $field.removeClass("selectedField");
            }

            $field.unbind('click').click($.proxy(self._clickField, self));
        });

        while (this._currentField.length < 6) {
            var $newNumber = Math.floor((Math.random() * 49) + 1);
            if ($.inArray($newNumber, this._currentField) === -1) {
                this._currentField.push($newNumber);
                $('.jsLottoBlockField').find("[data-number='" + $newNumber + "']").addClass("selectedField");
            }
        }

        this._updateBlockInfoCount(this._currentField.length);

        $('.jsLottoBlockSubmitt :submit').removeAttr('disabled');
        this._disabledSubmit = false;
    },
    _resetBlockField: function() {
        this._currentField = [];

        var self = this;
        $('.jsLottoBlockFields').each(function(index, field) {
            var $field = $(field);
            if ($field.hasClass("selectedField")) {
                $field.removeClass("selectedField");
            }

            $field.unbind('click').click($.proxy(self._clickField, self));
        });

        this._updateBlockInfoCount(0);

        $('.jsLottoBlockSubmitt :submit').removeAttr('disabled');
        this._disabledSubmit = false;
    },
    _updateTickets: function() {
        var self = this;
        $("div[data-block='" + this._activTicket + "']").each(function(index, field) {
            var $field = $(field);
            var $number = $field.data('number');

            if ($.inArray($number, self._selectedTickets[self._activTicket]) !== -1) {
                $field.addClass('selectedField');
            } else {
                $field.removeClass('selectedField');
            }
        });
    },
    _updateBlockInfoCount: function(countNumber) {
        var $count = 6 - countNumber;
        $('.jsLottoBlockInfoCount').find('span').html($count);
    },
    _submitButton: function() {
        this._proxy.setOption('data', {
            actionName: 'insertTickets',
            className: 'wcf\\data\\lotto\\ticket\\LottoTicketAction',
            parameters: {
                tickets: this._selectedTickets
            }
        });

        this._proxy.sendRequest();
    },
    _checkedSubmitButton: function() {
        var counter = 0;

        $.each(this._selectedTickets, function(index, value) {
            if (typeof value !== "undefined" && value.length === 6) {
                counter++;
            }
        });

        if (counter > 0) {
            $('#jsLottoSubmit').removeAttr('disabled');
        } else {
            $('#jsLottoSubmit').attr('disabled', 'disabled');
        }
    },
    _success: function(data, textStatus, jqXHR) {
        switch (data.actionName) {
            case 'insertTickets':
                var $notification = new WCF.System.Notification(WCF.Language.get('wcf.lotto.success.lottoAdd'));
                $notification.show();

                window.setTimeout('location.href="index.php/Lotto/"', 2000);
                break;
        }
    }
});

WCF.Lotto.TicketOverview = Class.extend({
    _containerSelector: null,
    init: function(containerSelector) {
        this._containerSelector = containerSelector;

        this._initElements();
    },
    _initElements: function() {
        var self = this;
        $(this._containerSelector).each(function(index, container) {
            $(container).click($.proxy(self._click, self));
        });
    },
    _click: function(event) {
        var $target = $(event.currentTarget);
        var $ticketID = $target.data("objectID");
        
        window.location = $target.data('link');
        return;
    }
});