<?php

namespace wcf\data\lotto\ticket;

use wcf\data\AbstractDatabaseObjectAction;
use wcf\data\lotto\stats\LottoStats;
use wcf\data\lotto\stats\LottoStatsAction;
use wcf\data\option\OptionEditor;
use wcf\data\user\jcoins\statement\UserJcoinsStatementAction;
use wcf\system\cache\builder\OptionCacheBuilder;
use wcf\system\exception\NamedUserException;
use wcf\system\user\activity\event\UserActivityEventHandler;
use wcf\system\WCF;

/**
 * @author      MaDa-Network
 * @copyright	2015 - MaDa-Network <http://mada-network.de>
 * @license	CC BY-ND 4.0 <http://creativecommons.org/licenses/by-nd/4.0/deed.de>
 * @package	de.madanetwork.lotto
 */
class LottoTicketAction extends AbstractDatabaseObjectAction {

    protected $className = 'wcf\data\lotto\ticket\LottoTicketEditor';
    protected $tickets = array();

    public function validateInsertTickets() {
        $this->tickets = $this->parameters['tickets'];

        foreach ($this->tickets as $key => $value) {
            if (empty($value))
                unset($this->tickets[$key]);
        }

        if (count($this->tickets) <= 1) {
            if ((WCF::getUser()->jCoinsBalance - JCOINS_RECEIVECOINS_LOTTOADD) < 0) {
                throw new NamedUserException(WCF::getLanguage()->getDynamicVariable('wcf.lotto.error.insertTickets.notEnoughCoins', array('countTickets' => count($this->tickets))));
            }
        } else {
            $jCoins = WCF::getUser()->jCoinsBalance - JCOINS_RECEIVECOINS_LOTTOADD;
            $counts = $jCoins - ((count($this->tickets) - 1) * JCOINS_RECEIVECOINS_LOTTOADDADDITIONALFIELD);

            if ($counts < 0) {
                throw new NamedUserException(WCF::getLanguage()->getDynamicVariable('wcf.lotto.error.insertTickets.notEnoughCoins', array('countTickets' => count($this->tickets))));
            }
        }
    }

    public function insertTickets() {
        $sum = 0;
        if (count($this->tickets) == 1) {
            $sum = JCOINS_RECEIVECOINS_LOTTOADD;
        } else {
            $sum = ((count($this->tickets) - 1) * JCOINS_RECEIVECOINS_LOTTOADDADDITIONALFIELD) + JCOINS_RECEIVECOINS_LOTTOADD;
        }

        $objectAction = new LottoTicketAction(array(), 'create', array('data' => array(
                'userID' => WCF::getUser()->userID,
                'ticketNumber' => $this->createTicketNumber(),
                'created' => TIME_NOW,
                'playCount' => 1,
                'costCoins' => $sum
        )));
        $returnValues = $objectAction->executeAction();
        $ticket = $returnValues['returnValues'];


        $sql = "INSERT INTO wcf" . WCF_N . "_lotto_ticket_field
                            (ticketID, fieldID, number1, number2, number3, number4, number5, number6)
                VALUES      (?, ?, ?, ?, ?, ?, ?, ?)";
        $statement = WCF::getDB()->prepareStatement($sql);

        $counters = array();
        foreach ($this->tickets as $fieldID => $ticketFields) {
            $statement->execute(array($ticket->ticketID, $fieldID, $ticketFields[0], $ticketFields[1], $ticketFields[2], $ticketFields[3], $ticketFields[4], $ticketFields[5]));

            foreach ($ticketFields as $number) {
                $key = "number$number";

                if (!isset($counters[$key]))
                    $counters[$key] = 0;
                $counters[$key] ++;
            }
        }

        if (!empty($counters)) {
            $statsAction = new LottoStatsAction(array(LottoStats::LOTTO_STATS_VOTED), 'update', array('counters' => $counters));
            $statsAction->executeAction();
        }

        $this->updateUserCoins($sum);

        UserActivityEventHandler::getInstance()->fireEvent('de.madanetwork.lotto.recentActivityEvent.lottoAdd', $ticket->ticketID);

        return $ticket;
    }

    protected function updateUserCoins($sum) {
        $statementAction = new UserJcoinsStatementAction(array(), 'create', array(
            'data' => array(
                'reason' => WCF::getLanguage()->getDynamicVariable('wcf.lotto.insertTickets.jCoinsReason', array('countTickets' => count($this->tickets))),
                'sum' => -$sum,
                'userID' => WCF::getUser()->userID
            ),
            'changeBalance' => 1
        ));
        $statementAction->validateAction();
        $statementAction->executeAction();

        $newJackpot = floor(LOTTO_CURRENT_JACKPOT + ($sum * 50 / 100));
        $sql = "UPDATE	wcf" . WCF_N . "_option
                SET	optionValue = ?
                WHERE	optionName = ?";
        $statement = WCF::getDB()->prepareStatement($sql);
        $statement->execute(array($newJackpot, 'lotto_current_jackpot'));

        OptionCacheBuilder::getInstance()->reset();
        OptionEditor::rebuild();
    }

    protected function createTicketNumber() {
        $newTicketNumer = '';

        while (strlen($newTicketNumer) < 10) {
            $rnd = rand(0, 9);
            $newTicketNumer .= $rnd;
        }
        
        $lottoTicketsList = new LottoTicketList();
        $lottoTicketsList->getConditionBuilder()->add('lotto_ticket.ticketNumber = ?', array($newTicketNumer));
        $count = $lottoTicketsList->countObjects();
        
        if ($count) {
            $newTicketNumer = $this->createTicketNumber();
        }
        
        return $newTicketNumer;
    }

}
