<?php

namespace wcf\page;

use wcf\data\lotto\ticket\LottoTicket;
use wcf\system\dashboard\DashboardHandler;
use wcf\system\exception\IllegalLinkException;
use wcf\system\user\collapsible\content\UserCollapsibleContentHandler;
use wcf\system\WCF;

/**
 * @author      MaDa-Network
 * @copyright	2015 - MaDa-Network <http://mada-network.de>
 * @license	CC BY-ND 4.0 <http://creativecommons.org/licenses/by-nd/4.0/deed.de>
 * @package	de.madanetwork.lotto
 */
class LottoTicketPage extends AbstractPage {
    public $activeMenuItem = 'wcf.header.menu.lotto.stats';
    public $neededModules = array('MODULE_LOTTO');
    public $ticket = null;
    public $ticketID = 0;
    public $additionalNumber = 0;
    public $fields = array();
    public $draws = array();
    public $winClassData = array();

    public function readParameters() {
        parent::readParameters();

        if (isset($_REQUEST['id'])) $this->ticketID = intval($_REQUEST['id']);

        $this->ticket = new LottoTicket($this->ticketID);
        if (!$this->ticket->ticketID || !$this->ticket->userID == WCF::getUser()->userID) {
            throw new IllegalLinkException();
        }

        WCF::getBreadcrumbs()->remove(0);
    }

    public function readData() {
        parent::readData();

        $fieldDatas = $this->ticket->getFields();
        foreach ($fieldDatas as $fieldData) {
            $this->fields[$fieldData['fieldID']][] = $fieldData['number1'];
            $this->fields[$fieldData['fieldID']][] = $fieldData['number2'];
            $this->fields[$fieldData['fieldID']][] = $fieldData['number3'];
            $this->fields[$fieldData['fieldID']][] = $fieldData['number4'];
            $this->fields[$fieldData['fieldID']][] = $fieldData['number5'];
            $this->fields[$fieldData['fieldID']][] = $fieldData['number6'];
        }

        if ($this->ticket->drawID) {
            $drawData = $this->ticket->getDraw();
            $this->draws[] = $drawData->number1;
            $this->draws[] = $drawData->number2;
            $this->draws[] = $drawData->number3;
            $this->draws[] = $drawData->number4;
            $this->draws[] = $drawData->number5;
            $this->draws[] = $drawData->number6;

            $this->additionalNumber = $drawData->additionalNumber;
            $this->winClassData = $drawData->getWinClassData();
        }
    }

    public function assignVariables() {
        parent::assignVariables();

        DashboardHandler::getInstance()->loadBoxes('de.madanetwork.lotto.LottoTicketPage', $this);

        WCF::getTPL()->assign(array(
            'ticket' => $this->ticket,
            'ticketID' => $this->ticketID,
            'fields' => $this->fields,
            'draws' => $this->draws,
            'additionalNumber' => $this->additionalNumber,
            'winClassData' => $this->winClassData,
            'sidebarCollapsed' => UserCollapsibleContentHandler::getInstance()->isCollapsed('com.woltlab.wcf.collapsibleSidebar', 'de.madanetwork.lotto.LottoTicketPage'),
            'sidebarName' => 'de.madanetwork.lotto.LottoTicketPage'
        ));
    }
}