<?php

namespace wcf\system\cronjob;

use DateInterval;
use DateTime;
use DateTimeZone;
use wcf\data\cronjob\Cronjob;
use wcf\data\lotto\draw\LottoDrawAction;
use wcf\data\lotto\stats\LottoStats;
use wcf\data\lotto\stats\LottoStatsAction;
use wcf\data\lotto\ticket\LottoTicket;
use wcf\data\lotto\ticket\LottoTicketAction;
use wcf\data\lotto\ticket\LottoTicketList;
use wcf\data\option\OptionEditor;
use wcf\data\user\jcoins\statement\UserJcoinsStatementAction;
use wcf\system\cache\builder\LottoLastWinnersCacheBuilder;
use wcf\system\cache\builder\OptionCacheBuilder;
use wcf\system\user\notification\object\LottoWinNotificationObject;
use wcf\system\user\notification\UserNotificationHandler;
use wcf\system\WCF;
use wcf\util\MathUtil;

/**
 * @author      MaDa-Network
 * @copyright	2015 - MaDa-Network <http://mada-network.de>
 * @license	CC BY-ND 4.0 <http://creativecommons.org/licenses/by-nd/4.0/deed.de>
 * @package	de.madanetwork.lotto
 */
class NextLotteryDrawCronjob extends AbstractCronjob {

    protected $additionalNumber = 0;
    protected $drawID = 0;
    protected $draw = array();
    protected $tickets = array();
    protected $fields = array();
    protected $hit = array();
    protected $winClass = array('1' => array(), '2' => array(), '3' => array(), '4' => array(), '5' => array(), '6' => array(), '7' => array(), '8' => array(), '9' => array());
    protected $jackpot = array('current' => 0, 'new' => 0);

    public function execute(Cronjob $cronjob) {
        parent::execute($cronjob);

        if (!MODULE_LOTTO)
            return;

        if (NEXT_LOTTERY_DRAW > TIME_NOW)
            return;

        $this->getLottery();
    }

    protected function getLottery() {
        self::loadTickets();
        self::newDraw();
        self::draw();
        self::matchs();
        self::updateStats();
        self::updateOptions();
        self::resetCache();
    }

    protected function loadTickets() {
        $lottoTicketsList = new LottoTicketList();
        $lottoTicketsList->getConditionBuilder()->add('lotto_ticket.playCount > ?', array(0));
        $lottoTicketsList->readObjects();
        $this->tickets = $lottoTicketsList->getObjects();

        foreach ($this->tickets as $tickets) {
            foreach ($tickets->getFields() as $fields) {
                $this->fields[$fields['ticketID']][$fields['fieldID']][] = $fields['number1'];
                $this->fields[$fields['ticketID']][$fields['fieldID']][] = $fields['number2'];
                $this->fields[$fields['ticketID']][$fields['fieldID']][] = $fields['number3'];
                $this->fields[$fields['ticketID']][$fields['fieldID']][] = $fields['number4'];
                $this->fields[$fields['ticketID']][$fields['fieldID']][] = $fields['number5'];
                $this->fields[$fields['ticketID']][$fields['fieldID']][] = $fields['number6'];
            }
        }
    }

    protected function newDraw() {
        while (count($this->draw) < 6) {
            $newNumber = MathUtil::getRandomValue(1, 49);

            if (!in_array($newNumber, $this->draw)) {
                $this->draw[] = $newNumber;
            }
        }

        $this->additionalNumber = MathUtil::getRandomValue(0, 9);

        $objectAction = new LottoDrawAction(array(), 'create', array('data' => array(
                'time' => TIME_NOW,
                'number1' => $this->draw[0],
                'number2' => $this->draw[1],
                'number3' => $this->draw[2],
                'number4' => $this->draw[3],
                'number5' => $this->draw[4],
                'number6' => $this->draw[5],
                'additionalNumber' => $this->additionalNumber
        )));
        $return = $objectAction->executeAction();
        $returnValues = $return['returnValues'];
        $this->drawID = $returnValues->drawID;
    }

    protected function draw() {
        foreach ($this->tickets as $ticket) {
            if (!isset($this->fields[$ticket->ticketID]))
                continue;

            $additional = false;

            $ticketLastNumber = substr($ticket->ticketNumber, -1, 1);
            if ($ticketLastNumber === $this->additionalNumber)
                $additional = true;

            foreach ($this->fields[$ticket->ticketID] as $fieldID => $fieldData) {
                foreach ($fieldData as $number) {
                    if (in_array($number, $this->draw)) {
                        $this->hit[$ticket->ticketID][$fieldID][] = $number;
                    }
                }

                if (isset($this->hit[$ticket->ticketID][$fieldID]) && (count($this->hit[$ticket->ticketID][$fieldID]) >= 2 && count($this->hit[$ticket->ticketID][$fieldID]) < 6)) {
                    switch (count($this->hit[$ticket->ticketID][$fieldID])) {
                        case 2:
                            if ($additional) {
                                $this->winClass[9][] = $ticket->ticketID . '.' . $fieldID;
                            }
                            break;
                        case 3:
                            if ($additional) {
                                $this->winClass[7][] = $ticket->ticketID . '.' . $fieldID;
                            } else {
                                $this->winClass[8][] = $ticket->ticketID . '.' . $fieldID;
                            }
                            break;
                        case 4:
                            if ($additional) {
                                $this->winClass[5][] = $ticket->ticketID . '.' . $fieldID;
                            } else {
                                $this->winClass[6][] = $ticket->ticketID . '.' . $fieldID;
                            }
                            break;
                        case 5:
                            if ($additional) {
                                $this->winClass[3][] = $ticket->ticketID . '.' . $fieldID;
                            } else {
                                $this->winClass[4][] = $ticket->ticketID . '.' . $fieldID;
                            }
                            break;
                        case 6:
                            if ($additional) {
                                $this->winClass[1][] = $ticket->ticketID . '.' . $fieldID;
                            } else {
                                $this->winClass[2][] = $ticket->ticketID . '.' . $fieldID;
                            }
                            break;
                    }
                }
            }

            $ticketIDs[] = $ticket->ticketID;
        }

        if (!empty($ticketIDs)) {
            $sql = "UPDATE	wcf" . WCF_N . "_lotto_ticket
                    SET         playCount = playCount - 1,
                                drawID = " . $this->drawID . "
                    WHERE	ticketID = ?";
            $statement = WCF::getDB()->prepareStatement($sql);

            WCF::getDB()->beginTransaction();
            foreach ($ticketIDs as $ticketID) {
                $statement->execute(array($ticketID));
            }
            WCF::getDB()->commitTransaction();
        }
    }

    protected function matchs() {
        $messageUser = array();
        $winCoins = self::readWinCoins();

        $sql = "UPDATE  wcf" . WCF_N . "_lotto_ticket_field
                SET     winCoins = ?
                WHERE   ticketID = ?
                    AND fieldID = ?";
        $updateLottoField = WCF::getDB()->prepareStatement($sql);

        $sql = "INSERT INTO     wcf" . WCF_N . "_lotto_draw_winclass
                                (drawID, winClass, pro, counts)
                VALUES          (?, ?, ?, ?)";
        $insertDrawWinClass = WCF::getDB()->prepareStatement($sql);

        for ($i = 1; $i <= 9; $i++) {
            $pro = $winCoins[$i];

            if (count($this->winClass[$i]) > 0) {
                $pro = floor($winCoins[$i] / count($this->winClass[$i]));

                foreach ($this->winClass[$i] as $value) {
                    $ex = explode(".", $value);
                    $ticketID = $ex[0];
                    $fieldID = $ex[1];

                    $userID = $this->tickets[$ticketID]->userID;

                    if (!isset($messageUser[$userID][$ticketID]['sum']))
                        $messageUser[$userID][$ticketID]['sum'] = 0;
                    $messageUser[$userID][$ticketID]['sum'] += $pro;

                    $updateLottoField->execute(array($pro, $ticketID, $fieldID));
                }
            }

            $insertDrawWinClass->execute(array($this->drawID, $i, $pro, count($this->winClass[$i])));
        }

        if (!empty($messageUser)) {
            $sum = 0;
            foreach ($messageUser as $userID => $tickets) {
                foreach ($tickets as $ticketID => $value) {
                    $objectAction = new LottoTicketAction(array($ticketID), 'update', array('data' => array(
                            'winCoins' => $value['sum']
                    )));
                    $objectAction->executeAction();

                    if ($value['sum'] != 0) {
                        $statementAction = new UserJcoinsStatementAction(array(), 'create', array('data' => array(
                                'reason' => WCF::getLanguage()->getDynamicVariable('wcf.lotto.lottoWin.ticket', array('ticketID' => $ticketID)),
                                'sum' => $value['sum'],
                                'userID' => $userID
                            ),
                            'changeBalance' => 1
                        ));
                        $statementAction->validateAction();
                        $statementAction->executeAction();
                    }

                    $sum += $value['sum'];
                    UserNotificationHandler::getInstance()->fireEvent('lottoWin', 'de.madanetwork.lotto.win.notification', new LottoWinNotificationObject(new LottoTicket($ticketID)), array($userID));
                }
            }

            $newJackpot = $this->jackpot['current'] - $sum;
            $this->jackpot['new'] = $newJackpot;
        }
    }

    protected function readWinCoins() {
        $winCoins = array();
        $this->jackpot['current'] = $this->jackpot['new'] = intval(LOTTO_CURRENT_JACKPOT);

        $winCoins[9] = floor(LOTTO_WINCLASS9);
        if (!$winCoins[9])
            $winCoins[9] = 1;
        $jackpotRemoveWinClass9 = $winCoins[9] * count($this->winClass[9]);

        $this->jackpot['current'] = $this->jackpot['current'] - $jackpotRemoveWinClass9;

        $winCoins[1] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS1) / 100);
        $jackpotRemoveWinClass1 = $winCoins[1] * count($this->winClass[1]);

        $this->jackpot['current'] = $this->jackpot['current'] - $jackpotRemoveWinClass1;

        $winCoins[2] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS2) / 100);
        $winCoins[3] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS3) / 100);
        $winCoins[4] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS4) / 100);
        $winCoins[5] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS5) / 100);
        $winCoins[6] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS6) / 100);
        $winCoins[7] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS7) / 100);
        $winCoins[8] = floor($this->jackpot['current'] * floatval(LOTTO_WINCLASS8) / 100);

        return $winCoins;
    }

    protected function updateStats() {
        $counters = array();
        foreach ($this->draw as $draw) {
            $key = "number$draw";
            $counters[$key] = 1;
        }

        $statsAction = new LottoStatsAction(array(LottoStats::LOTTO_STATS_DRAW), 'update', array('counters' => $counters));
        $statsAction->executeAction();
    }

    protected function updateOptions() {
        $playDate = explode("\n", LOTTO_PLAY_DATE);
        $date = new DateTime('NOW');
        $date->setTimezone(new DateTimeZone(TIMEZONE));
        $date->setTime(LOTTO_PLAY_TIME, 0, 0);

        if (count($playDate)) {
            while (true) {
                $date->add(new DateInterval('P1D'));

                if (in_array($date->format('N'), $playDate))
                    break;
            }
        } else {
            $date->add(new DateInterval('P1D'));
        }
        
        $sql = "UPDATE	wcf" . WCF_N . "_option
                SET	optionValue = ?
                WHERE	optionName = ?";
        $statement = WCF::getDB()->prepareStatement($sql);
        $statement->execute(array($date->getTimestamp(), 'next_lottery_draw'));
        $statement->execute(array($this->jackpot['new'], 'lotto_current_jackpot'));
    }

    protected function resetCache() {
        OptionCacheBuilder::getInstance()->reset();
        OptionEditor::rebuild();

        LottoLastWinnersCacheBuilder::getInstance()->reset();
    }

}
